@@ -53,6 +53,8 @@ from pitivi.utils.widgets import GstElementSettingsWidget
 
 AUDIO_EFFECTS_CATEGORIES = ()
 
+ALLOWED_ONLY_ONCE_EFFECTS = ['videoflip']
+
 VIDEO_EFFECTS_CATEGORIES = (
     (_("Colors"), (
         # Mostly "serious" stuff that relates to correction/adjustments
@@ -127,6 +129,19 @@ BLACKLISTED_EFFECTS = ["colorconvert", "coglogoinsert", "festival",
 
 BLACKLISTED_PLUGINS = []
 
+HIDDEN_EFFECTS = [
+    # Overlaying an image onto a video stream can already be done.
+    "gdkpixbufoverlay"]
+
+GlobalSettings.addConfigSection('effect-library')
+
+(COL_NAME_TEXT,
+ COL_DESC_TEXT,
+ COL_EFFECT_TYPE,
+ COL_EFFECT_CATEGORIES,
+ COL_ELEMENT_NAME,
+ COL_ICON) = list(range(6))
+
 ICON_WIDTH = 48 + 2 * 6  # 48 pixels, plus a margin on each side
 
 
@@ -149,7 +164,6 @@ class EffectInfo(object):
     @property
     def icon(self):
         pixdir = os.path.join(get_pixmap_dir(), "effects")
-        icon = None
         try:
             # We can afford to scale the images here, the impact is negligible
             icon = GdkPixbuf.Pixbuf.new_from_file_at_size(
@@ -161,6 +175,22 @@ class EffectInfo(object):
                 os.path.join(pixdir, "defaultthumbnail.svg"))
         return icon
 
+    @property
+    def bin_description(self):
+        """Gets the bin description which defines this effect."""
+        if self.effect_name.startswith("gl"):
+            return "glupload ! %s ! gldownload" % self.effect_name
+        else:
+            return self.effect_name
+
+    @staticmethod
+    def name_from_bin_description(bin_description):
+        """Gets the name of the effect defined by the `bin_description`."""
+        if bin_description.startswith("glupload"):
+            return bin_description.split("!")[1].strip()
+        else:
+            return bin_description
+
 
 class EffectsManager(object):
     """Keeps info about effects and their categories.
@@ -176,8 +206,9 @@ class EffectsManager(object):
         self.audio_effects = []
         self._effects = {}
 
-        uselessWords = [_("Video"), _("Audio"), _("Audio").lower(), _("effect")]
-        uselessRe = re.compile(" |".join(uselessWords))
+        useless_words = ["Video", "Audio", "audio", "effect",
+                         _("Video"), _("Audio"), _("Audio").lower(), _("effect")]
+        uselessRe = re.compile(" |".join(useless_words))
 
         factories = Gst.Registry.get().get_feature_list(Gst.ElementFactory)
         longnames = set()
@@ -219,15 +250,16 @@ class EffectsManager(object):
                                 description=factory.get_description())
             self._effects[name] = effect
 
-    def getInfo(self, name):
+    def getInfo(self, bin_description):
         """Gets the info for an effect which can be applied.
 
         Args:
-            name (str): The bin_description identifying the effect.
+            bin_description (str): The bin_description defining the effect.
 
         Returns:
             EffectInfo: The info corresponding to the name, or None.
         """
+        name = EffectInfo.name_from_bin_description(bin_description)
         return self._effects.get(name)
 
     def _getEffectCategories(self, effect_name):
@@ -274,20 +306,6 @@ class EffectsManager(object):
 
 # ----------------------- UI classes to manage effects -------------------------#
 
-HIDDEN_EFFECTS = [
-    # Overlaying an image onto a video stream can already be done.
-    "gdkpixbufoverlay",
-    "frei0r-filter-scale0tilt"]
-
-GlobalSettings.addConfigSection('effect-library')
-
-(COL_NAME_TEXT,
- COL_DESC_TEXT,
- COL_EFFECT_TYPE,
- COL_EFFECT_CATEGORIES,
- COL_ELEMENT_NAME,
- COL_ICON) = list(range(6))
-
 
 class EffectListWidget(Gtk.Box, Loggable):
     """Widget for listing effects."""
@@ -315,16 +333,17 @@ class EffectListWidget(Gtk.Box, Loggable):
         # Store
         self.storemodel = Gtk.ListStore(
             str, str, int, object, str, GdkPixbuf.Pixbuf)
+        self.storemodel.set_sort_column_id(
+            COL_NAME_TEXT, Gtk.SortType.ASCENDING)
 
-        self.view = Gtk.TreeView(model=self.storemodel)
+        # Create the filter for searching the storemodel.
+        self.model_filter = self.storemodel.filter_new()
+        self.model_filter.set_visible_func(self._setRowVisible, data=None)
+
+        self.view = Gtk.TreeView(model=self.model_filter)
         self.view.props.headers_visible = False
         self.view.get_selection().set_mode(Gtk.SelectionMode.SINGLE)
 
-        # Create the filterModel for searching the storemodel
-        self.modelFilter = self.storemodel.filter_new()
-        self.modelFilter.set_visible_func(self._setRowVisible, data=None)
-        self.view.set_model(self.modelFilter)
-
         icon_col = Gtk.TreeViewColumn()
         icon_col.set_spacing(SPACING)
         icon_col.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
@@ -413,8 +432,6 @@ class EffectListWidget(Gtk.Box, Loggable):
                                     effect_info.categories,
                                     name,
                                     effect_info.icon])
-        self.storemodel.set_sort_column_id(
-            COL_NAME_TEXT, Gtk.SortType.ASCENDING)
 
     def populate_categories_widget(self):
         self.categoriesWidget.get_model().clear()
@@ -434,7 +451,6 @@ class EffectListWidget(Gtk.Box, Loggable):
     def _dndDragDataGetCb(self, unused_view, drag_context, selection_data, unused_info, unused_timestamp):
         data = bytes(self.getSelectedEffect(), "UTF-8")
         selection_data.set(drag_context.list_targets()[0], 0, data)
-        return True
 
     def _rowUnderMouseSelected(self, view, event):
         result = view.get_path_at_pos(int(event.x), int(event.y))
@@ -476,7 +492,7 @@ class EffectListWidget(Gtk.Box, Loggable):
         if self._draggedItems:
             return self._draggedItems
         model, rows = self.view.get_selection().get_selected_rows()
-        path = self.modelFilter.convert_path_to_child_path(rows[0])
+        path = self.model_filter.convert_path_to_child_path(rows[0])
         return self.storemodel[path][COL_ELEMENT_NAME]
 
     def _toggleViewTypeCb(self, widget):
@@ -495,13 +511,13 @@ class EffectListWidget(Gtk.Box, Loggable):
         else:
             self._effectType = AUDIO_EFFECT
         self.populate_categories_widget()
-        self.modelFilter.refilter()
+        self.model_filter.refilter()
 
     def _categoryChangedCb(self, unused_combobox):
-        self.modelFilter.refilter()
+        self.model_filter.refilter()
 
     def _searchEntryChangedCb(self, unused_entry):
-        self.modelFilter.refilter()
+        self.model_filter.refilter()
 
     def _searchEntryIconClickedCb(self, entry, unused, unused1):
         entry.set_text("")
@@ -588,5 +604,4 @@ class EffectsPropertiesManager:
             with self.app.action_log.started("Effect property change",
                                              CommitTimelineFinalizingAction(pipeline)):
                 effect.set_child_property(prop.name, value)
-            pipeline.commit_timeline()
             self._current_element_values[prop.name] = value
